@@ -95,7 +95,7 @@ except Exception, e:
 
 
 CONFIG = {
-    'zoom':       3,
+    'zoom':       2,
     'lat':        0.0,
     'lon':        0.0,
 
@@ -105,6 +105,13 @@ CONFIG = {
     'gpxfile':      '',
 
     'gpslock':      0,
+    
+    'Google - places':      1,
+    'Google - routesearch': 1,
+    'gpx track':            1,
+    'Results view':         1,
+    'POI':                  1,
+    'Google - geonames':    1
 
     }
 
@@ -172,79 +179,82 @@ class YosmapaOSD(gobject.GObject, osmgpsmap.GpsMapLayer):
         self.gdkdrawable = None
 
     def do_draw(self, gpsmap, gdkdrawable):
-        #return None
-        if self.make_drawing:
-            if timing_osd:
-                t = time.time()
-
-            self.size = gdkdrawable.get_size()
-            CONFIG['width'] = self.size[0]
-            CONFIG['height'] = self.size[1]
-            x = self.size[0]-64
-
-            cr = gdkdrawable.cairo_create()
-            self.gdkdrawable = gdkdrawable
-
-            cr.set_source_surface( self.pb_plus, x,10 )
-            cr.paint()
-
-            if gpsPresent:
-                if self.osmMapa.get_gpslock() == 1:
-                    cr.set_source_surface( self.pb_gps_on, x,80 )
-                else:
-                    cr.set_source_surface( self.pb_gps, x,80 )
+        if self.osmMapa.marker_draw_redraw_made == 0 and self.make_drawing:
+            print "\t\t\- YosmapaOSD do_draw"
+            #return None
+            
+            if self.make_drawing:
+                if timing_osd:
+                    t = time.time()
+    
+                self.size = gdkdrawable.get_size()
+                CONFIG['width'] = self.size[0]
+                CONFIG['height'] = self.size[1]
+                x = self.size[0]-64
+    
+                cr = gdkdrawable.cairo_create()
+                self.gdkdrawable = gdkdrawable
+    
+                cr.set_source_surface( self.pb_plus, x,10 )
                 cr.paint()
-
-                try:
-                    self.osmMapa.osm.props.gps_track_highlight_radius = (gps.accuracy/gps.acc_scale[self.osmMapa.osm.props.zoom])*10
-                except:
-                    self.osmMapa.osm.props.gps_track_highlight_radius = 0
-                    print "GPS manager: accurasy is insain (",self.accuracy,")"
-
-
-            cr.set_source_surface( self.pb_min, x,160 )
-            cr.paint()
-            cr.set_source_surface( self.pb_menu, x,self.size[1]-100 )
-            cr.paint()
-
-            if len(self.osmMapa.find_plugins) > 0:
-                if len(self.osmMapa.show_find_results) > 0:
-                    cr.set_source_surface( self.pb_clean, x+10,self.size[1]-160 )
-                else:
-                    cr.set_source_surface( self.pb_find, x+10,self.size[1]-160 )
+    
+                if gpsPresent:
+                    if self.osmMapa.get_gpslock() == 1:
+                        cr.set_source_surface( self.pb_gps_on, x,80 )
+                    else:
+                        cr.set_source_surface( self.pb_gps, x,80 )
+                    cr.paint()
+    
+                    try:
+                        self.osmMapa.osm.props.gps_track_highlight_radius = (gps.accuracy/gps.acc_scale[self.osmMapa.osm.props.zoom])*10
+                    except:
+                        self.osmMapa.osm.props.gps_track_highlight_radius = 0
+                        print "GPS manager: accurasy is insain (",self.accuracy,")"
+    
+    
+                cr.set_source_surface( self.pb_min, x,160 )
                 cr.paint()
-
-            plug_nr = 0
-            for plugin in self.osmMapa.plugins_list:
-                plugin_name = plugin[1]
-                if CONFIG[plugin_name] and plugin[2]:
-                    if len(plugin) == 3:
-                        plugin.append( plugin[0].get_menu_widgets(self) )
-                    if plugin[3] <> None:
-                        try:
-                            if plugin[0].ui_menu_left:
-                                cr.set_source_surface( plugin[2], 2, (plug_nr*self.plugin_ico_h)+self.plugin_ico_yoffset )
-                                cr.paint()
-                                plug_nr+=1
-                        except:
-                                cr.set_source_surface( plugin[2], 2, (plug_nr*self.plugin_ico_h)+self.plugin_ico_yoffset )
-                                cr.paint()
-                                plug_nr+=1
-
-
-            if self.osmMapa.osm.props.tiles_queued != 0:
-                str = "Downloading images (%i)" % self.osmMapa.osm.props.tiles_queued
-                cr.set_source_rgb(0.0, 0.0, 0.0)
-                cr.select_font_face("Georgia")
-                cr.set_font_size(15)
-                cr.move_to(10,15)
-                cr.show_text(str)
-            if timing_osd:
-                print "YosmapaOSD do_draw in:",(time.time()-t)
-
-            self.make_drawing = 0
-
-        return None
+                cr.set_source_surface( self.pb_menu, x,self.size[1]-100 )
+                cr.paint()
+    
+                if len(self.osmMapa.find_plugins) > 0:
+                    if len(self.osmMapa.show_find_results) > 0:
+                        cr.set_source_surface( self.pb_clean, x+10,self.size[1]-160 )
+                    else:
+                        cr.set_source_surface( self.pb_find, x+10,self.size[1]-160 )
+                    cr.paint()
+    
+                plug_nr = 0
+                for plugin in self.osmMapa.plugins_list:
+                    plugin_name = plugin[1]
+                    if CONFIG[plugin_name] and plugin[2]:
+                        if len(plugin) == 3:
+                            plugin.append( plugin[0].get_menu_widgets(self) )
+                        if plugin[3] <> None:
+                            try:
+                                if plugin[0].ui_menu_left:
+                                    cr.set_source_surface( plugin[2], 2, (plug_nr*self.plugin_ico_h)+self.plugin_ico_yoffset )
+                                    cr.paint()
+                                    plug_nr+=1
+                            except:
+                                    cr.set_source_surface( plugin[2], 2, (plug_nr*self.plugin_ico_h)+self.plugin_ico_yoffset )
+                                    cr.paint()
+                                    plug_nr+=1
+    
+    
+                if self.osmMapa.osm.props.tiles_queued != 0:
+                    str = "Downloading images (%i)" % self.osmMapa.osm.props.tiles_queued
+                    cr.set_source_rgb(0.0, 0.0, 0.0)
+                    cr.select_font_face("Georgia")
+                    cr.set_font_size(15)
+                    cr.move_to(10,15)
+                    cr.show_text(str)
+                if timing_osd:
+                    print "YosmapaOSD do_draw in:",(time.time()-t)
+    
+                self.make_drawing = 0
+            print "\t\t\- YosmapaOSD do_draw make it"
+            return None
 
     def do_render(self, gpsmap):
         self.make_drawing = 1
@@ -624,6 +634,8 @@ class UI(hildon.StackableWindow):
         self.window_rotate = -1
         self.set_autorotate_for_object(self)
         
+        self.osm_box = []
+        self.marker_draw_redraw_made = 0
         
         try:
             self.osm.set_center_and_zoom( CONFIG['lat'], CONFIG['lon'], CONFIG['zoom'] )
@@ -727,13 +739,14 @@ class UI(hildon.StackableWindow):
         print "recalculate_center_and_zoom ---- Done"
 
     def marker_draw(self,cr):
-
-        print "marker_draw"
+        print "\t\t\- UI marker_draw"
         self.clikable_clouds = []
         t_lt, t_ln, b_lt, b_ln = self.osm.get_bbox()
-        t_lt, t_ln, b_lt, b_ln = math.degrees(t_lt), math.degrees(t_ln), math.degrees(b_lt), math.degrees(b_ln)
+        osm_box = math.degrees(t_lt), math.degrees(t_ln), math.degrees(b_lt), math.degrees(b_ln)
+        t_lt, t_ln, b_lt, b_ln = osm_box 
         zoom =  self.osm.props.zoom
 
+        
         if len(self.show_find_results)>0:
             self.marker_clear_from_layer( self.markers )
             make_from = self.show_find_results
@@ -747,7 +760,7 @@ class UI(hildon.StackableWindow):
             #print "icon_clicked !",self.cliked_item
         else:
             icon_clicked = 0
-
+        
         for m in make_from:
             lt = m[0]
             ln = m[1]
@@ -759,6 +772,7 @@ class UI(hildon.StackableWindow):
                 self.clikable_icons.append( [x, y, x+xoff, y+yoff, m] )
 
                 if not m[5]:
+                    self.marker_draw_redraw_made = 1
                     m[5] = self.osm.image_add( lt, ln, m[4] )
                     if self.cliked_item == [] and m[3]:
                         self.cliked_item = [lt,ln]
@@ -776,13 +790,14 @@ class UI(hildon.StackableWindow):
                         self.draw_cloud_byll(cr, m)
             else:
                 if m[5]:
+                    self.marker_draw_redraw_made = 1
                     self.osm.image_remove(m[5])
                     m[5] = None
                     if icon_clicked:
                         if self.cliked_item[0] == m[0] and self.cliked_item[1] == m[1]:
                             self.cliked_item = []
-
-
+        
+        
     def marker_clear_from_layer(self, from_what):
         for m in from_what:
             if m[5]:
@@ -805,8 +820,18 @@ class UI(hildon.StackableWindow):
                     mark_tmp.append(m)
             self.markers = mark_tmp
 
-    def marker_add(self, marker):
-        self.markers.append( [marker[0], marker[1], marker[2], marker[3], marker[4], None, marker[6]] )
+    def marker_add(self, marker, add_distinct = False):
+        marker[5] = None
+        if add_distinct == False:
+            self.markers.append( marker )
+        else:
+            add = 1
+            for m in self.markers:
+                if m[0] == marker[0] and m[1] == marker[1] and m[2] == marker[2]:
+                    add = 0
+                    break
+            if add:
+                self.markers.append( marker )
 
     def marker_del(self, marker):
         mark_tmp = []
@@ -867,7 +892,7 @@ class UI(hildon.StackableWindow):
         return math.sqrt((lon0 - lon1)**2 + (lat0 - lat1)**2)
 
     def draw_arrow(self, gdkdrawable, lat, lon,rgba=[0.9,0.2,0.9,0.7], borderRgba=[0.0,0.4,0.4,0.3] ):
-
+        print "\t\t\- UI draw_arrow"
         cr = gdkdrawable.cairo_create()
         t_lt, t_ln, b_lt, b_ln = self.osm.get_bbox()
         t_lt, t_ln, b_lt, b_ln = math.degrees(t_lt), math.degrees(t_ln), math.degrees(b_lt), math.degrees(b_ln)
@@ -921,88 +946,90 @@ class UI(hildon.StackableWindow):
             str = lat
         self.draw_cloud(cr, x, y, str, font_size, Fifek)
     def draw_cloud(self, cr, x, y, str = None, font_size = 13, Fifek = True):
-
-        point = None
-        if type(str) == list:
-            point = str
-            str = point[3]
-
-        if str == "" or str == None:
-            return None
-        if x<0:
-            print "inwert"
-            x = CONFIG['width']+x
-        if y<0:
-            print "inwert"
-            y = CONFIG['height']+y
-
-        rlat, rlon = self.osm.get_co_ordinates(int(x), int(y))
-        cr.select_font_face("Georgia")
-        cr.set_font_size(font_size)
-        xbearing, ybearing, width, height, xadvance, yadvance = cr.text_extents( str )
-
-        line_height = height
-        multiline = 0
-        if str.find("\n") <> -1:
-            multiline = 1
-            tmp = str.split("\n")
+        if self.marker_draw_redraw_made == 0:
+            
+            print "\t\t\- UI draw_cloud"
+            point = None
+            if type(str) == list:
+                point = str
+                str = point[3]
+    
+            if str == "" or str == None:
+                return None
+            if x<0:
+                print "inwert"
+                x = CONFIG['width']+x
+            if y<0:
+                print "inwert"
+                y = CONFIG['height']+y
+    
+            rlat, rlon = self.osm.get_co_ordinates(int(x), int(y))
+            cr.select_font_face("Georgia")
+            cr.set_font_size(font_size)
+            xbearing, ybearing, width, height, xadvance, yadvance = cr.text_extents( str )
+    
             line_height = height
-            width = 0
-            height*=len(tmp)
-            for l in tmp:
-                xbearing, ybearing, width2, height2, xadvance, yadvance = cr.text_extents( l )
-                if width2> width:
-                    width = width2
-
-        y+=3
-
-        aspect = 1.0
-        corner_radius = 3.0
-        M_PI = 3.14159265
-
-        radius = corner_radius / aspect
-        degrees = M_PI / 180.0
-
-        cr.new_sub_path ()
-        if Fifek:
-            cr.line_to( x,y )
-            cr.line_to( x+5,y+10 )
-        x-=5
-        y+=10
-        height+=4
-        width+=4
-
-        cr.arc (x + width - radius, y + radius, radius, -90 * degrees, 0 * degrees)
-        cr.arc (x + width - radius, y + height - radius, radius, 0 * degrees, 90 * degrees)
-        cr.arc (x + radius, y + height - radius, radius, 90 * degrees, 180 * degrees)
-        cr.arc (x + radius, y + radius, radius, 180 * degrees, 270 * degrees)
-        cr.close_path ()
-
-        cr.set_source_rgb (1.0, 1.0, 1.0)
-        cr.fill_preserve ()
-        cr.set_source_rgb (0.0, 0.0, 0.0)
-        cr.set_line_width (1.0)
-        cr.stroke ()
-
-        if multiline:
-            line_nr = 0
-            for l in tmp:
-                cr.move_to(x+2,y+12+(line_nr*line_height))
-                cr.show_text(l)
-                line_nr+=1
-
-        else:
-            cr.move_to(x+2,y+line_height+1)
-            cr.show_text(str)
-
-        if Fifek:
-            if point:
-                point_data = point
+            multiline = 0
+            if str.find("\n") <> -1:
+                multiline = 1
+                tmp = str.split("\n")
+                line_height = height
+                width = 0
+                height*=len(tmp)
+                for l in tmp:
+                    xbearing, ybearing, width2, height2, xadvance, yadvance = cr.text_extents( l )
+                    if width2> width:
+                        width = width2
+    
+            y+=3
+    
+            aspect = 1.0
+            corner_radius = 3.0
+            M_PI = 3.14159265
+    
+            radius = corner_radius / aspect
+            degrees = M_PI / 180.0
+    
+            cr.new_sub_path ()
+            if Fifek:
+                cr.line_to( x,y )
+                cr.line_to( x+5,y+10 )
+            x-=5
+            y+=10
+            height+=4
+            width+=4
+    
+            cr.arc (x + width - radius, y + radius, radius, -90 * degrees, 0 * degrees)
+            cr.arc (x + width - radius, y + height - radius, radius, 0 * degrees, 90 * degrees)
+            cr.arc (x + radius, y + height - radius, radius, 90 * degrees, 180 * degrees)
+            cr.arc (x + radius, y + radius, radius, 180 * degrees, 270 * degrees)
+            cr.close_path ()
+    
+            cr.set_source_rgb (1.0, 1.0, 1.0)
+            cr.fill_preserve ()
+            cr.set_source_rgb (0.0, 0.0, 0.0)
+            cr.set_line_width (1.0)
+            cr.stroke ()
+    
+            if multiline:
+                line_nr = 0
+                for l in tmp:
+                    cr.move_to(x+2,y+12+(line_nr*line_height))
+                    cr.show_text(l)
+                    line_nr+=1
+    
             else:
-                point_data = str
-            self.clikable_clouds.append( [ x,y,x+width,y+height, float(math.degrees(rlat)), float(math.degrees(rlon)), point_data ] )
-
-        return True
+                cr.move_to(x+2,y+line_height+1)
+                cr.show_text(str)
+    
+            if Fifek:
+                if point:
+                    point_data = point
+                else:
+                    point_data = str
+                self.clikable_clouds.append( [ x,y,x+width,y+height, float(math.degrees(rlat)), float(math.degrees(rlon)), point_data ] )
+    
+            return True
 
     def point_test(self, lat, lon, x,y):
         return False
@@ -1087,9 +1114,10 @@ class UI(hildon.StackableWindow):
     def map_release_menu(self, xR, yR ):
         t = time.time()
 
-        accuracy = 10
-        if int(xR/accuracy) == int(self.cliked_at_xyt[0]/accuracy) and int(yR/accuracy) == int(self.cliked_at_xyt[1]/accuracy) and (t-0.5) > self.cliked_at_xyt[2]:
-            print "menu tab map show !"
+        accuracy = 19
+        dis_release = self.cal_dist(xR, yR, self.cliked_at_xyt[0], self.cliked_at_xyt[1])
+        print "menu tab map show !",dis_release
+        if dis_release< accuracy and (t-0.3) > self.cliked_at_xyt[2]:
 
             m = gtk.Menu()
             latc,lonc = self.get_may_current_ll()
@@ -1235,38 +1263,42 @@ class PluginsLayer(gobject.GObject, osmgpsmap.GpsMapLayer):
         self.make_draw = 1
 
     def do_draw(self, gpsmap, gdkdrawable):
+        print "\t\t\- PluginsLayer do_draw"
         if self.make_draw and CONFIG['work_work_work']:
-            if timing_plugins:
-                t = time.time()
-
             self.osmMapa.marker_draw(gdkdrawable.cairo_create())
-
-            if len( self.osmMapa.do_draw_plugins ) == 0:
-                make_from = self.plugins
-                add = 1
-            else:
-                make_from = self.osmMapa.do_draw_plugins
-                add = 0
-
-            for plugin in make_from:
-                if CONFIG[plugin[1]]:
-                    try:
-                        if timing_plugins:
-                            tp = time.time()
-                        plugin[0].do_draw(gpsmap, gdkdrawable)
-                        if timing_plugins:
-                            print "plugin ", plugin[1]," in ",(time.time()-tp)
-                        if add:
-                            self.osmMapa.do_draw_plugins.append(plugin)
-                    except Exception, e:
-                        print "PluginsLayer do_draw ERROR (",plugin[1],"):", e
-
-
-            if timing_plugins:
-                print "PluginsLayers in",(time.time()-t)
-            self.make_draw = 0
+            if self.osmMapa.marker_draw_redraw_made == 0:
+                print "\t\t\- PluginsLayer do_draw make it"
+                if timing_plugins:
+                    t = time.time()
+    
+    
+                if len( self.osmMapa.do_draw_plugins ) == 0:
+                    for plugin in self.plugins:
+                        if CONFIG[plugin[1]]:
+                            try:
+                                if timing_plugins:
+                                    tp = time.time()
+                                plugin[0].do_draw(gpsmap, gdkdrawable)
+                                if timing_plugins:
+                                    print "plugin ", plugin[1]," in ",(time.time()-tp)
+                                self.osmMapa.do_draw_plugins.append(plugin)
+                            except Exception, e:
+                                print "PluginsLayer do_draw ERROR (",plugin[1],"):", e
+                else:
+                    for plugin in self.osmMapa.do_draw_plugins:
+                        plugin[0].do_draw(gpsmap, gdkdrawable)           
+    
+                if timing_plugins:
+                    print "PluginsLayers in",(time.time()-t)
+                self.make_draw = 0
 
     def do_render(self, gpsmap):
+        t_lt, t_ln, b_lt, b_ln = self.osmMapa.osm.get_bbox()
+        t_lt, t_ln, b_lt, b_ln = math.degrees(t_lt), math.degrees(t_ln), math.degrees(b_lt), math.degrees(b_ln)
+        self.osmMapa.osm_box = t_lt, t_ln, b_lt, b_ln
+        print "\t- PluginsLayer do_render"
+        print "\t",self.osmMapa.osm.props.tiles_queued, 'tiles queued'
+        self.osmMapa.marker_draw_redraw_made = 0
         self.make_draw = 1
         pass
 
